<?php

namespace Restaurant\Table\Policies;

use Litepie\User\Contracts\UserPolicy;
use Restaurant\Table\Models\Table;

class TablePolicy
{

    /**
     * Determine if the given user can view the table.
     *
     * @param UserPolicy $user
     * @param Table $table
     *
     * @return bool
     */
    public function view(UserPolicy $user, Table $table)
    {
        if ($user->canDo('table.table.view') && $user->isAdmin()) {
            return true;
        }

        return $table->user_id == user_id() && $table->user_type == user_type();
    }

    /**
     * Determine if the given user can create a table.
     *
     * @param UserPolicy $user
     * @param Table $table
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('table.table.create');
    }

    /**
     * Determine if the given user can update the given table.
     *
     * @param UserPolicy $user
     * @param Table $table
     *
     * @return bool
     */
    public function update(UserPolicy $user, Table $table)
    {
        if ($user->canDo('table.table.edit') && $user->isAdmin()) {
            return true;
        }

        return $table->user_id == user_id() && $table->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given table.
     *
     * @param UserPolicy $user
     * @param Table $table
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Table $table)
    {
        return $table->user_id == user_id() && $table->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given table.
     *
     * @param UserPolicy $user
     * @param Table $table
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Table $table)
    {
        if ($user->canDo('table.table.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given table.
     *
     * @param UserPolicy $user
     * @param Table $table
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Table $table)
    {
        if ($user->canDo('table.table.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
